<script setup lang="ts">
import type { Recordable } from '@vben/types';

import { Page, useVbenDrawer, type VbenFormProps } from '@vben/common-ui';
import { eachTree, getVxePopupContainer } from '@vben/utils';

import { Popconfirm, Space } from 'ant-design-vue';

import { useVbenVxeGrid, type VxeGridProps } from '#/adapter/vxe-table';
import { delEquType, equTypeExport, listEquType } from '#/api/eims/equ-type';
import { commonDownloadExcel } from '#/utils/file/download';

import { columns, querySchema } from './data';
import equTypeDrawer from './equ-type-drawer.vue';

/**
 * 不要问为什么有两个根节点 v-if会控制只会渲染一个
 */

const formOptions: VbenFormProps = {
  commonConfig: {
    labelWidth: 80,
    componentProps: {
      allowClear: true
    }
  },
  schema: querySchema(),
  wrapperClass: 'grid-cols-1 md:grid-cols-2 lg:grid-cols-3 xl:grid-cols-4'
};

const gridOptions: VxeGridProps = {
  columns,
  height: 'auto',
  keepSource: true,
  pagerConfig: {
    enabled: false
  },
  proxyConfig: {
    ajax: {
      query: async (_, formValues = {}) => {
        const resp = await listEquType({
          ...formValues
        });
        return { rows: resp };
      }
    }
  },
  rowConfig: {
    isHover: true,
    keyField: 'equTypeId'
  },
  /**
   * 开启虚拟滚动
   * 数据量小可以选择关闭
   * 如果遇到样式问题(空白、错位 滚动等)可以选择关闭虚拟滚动
   */
  scrollY: {
    enabled: true,
    gt: 0
  },
  treeConfig: {
    parentField: 'parentId',
    rowField: 'equTypeId',
    // 自动转换为tree 由vxe处理 无需手动转换
    transform: true
  },
  id: 'equ-type-index'
};

const [BasicTable, tableApi] = useVbenVxeGrid({
  formOptions,
  gridOptions,
  gridEvents: {
    cellDblclick: (e: any) => {
      const { row = {} } = e;
      if (!row?.children) {
        return;
      }
      const isExpanded = row?.expand;
      tableApi.grid.setTreeExpand(row, !isExpanded);
      row.expand = !isExpanded;
    },
    // 需要监听使用箭头展开的情况 否则展开/折叠的数据不一致
    toggleTreeExpand: (e: any) => {
      const { row = {}, expanded } = e;
      row.expand = expanded;
    }
  }
});
const [EquTypeDrawer, drawerApi] = useVbenDrawer({
  connectedComponent: equTypeDrawer
});

function handleAdd() {
  drawerApi.setData({});
  drawerApi.open();
}

function handleSubAdd(row: Recordable<any>) {
  const { equTypeId } = row;
  drawerApi.setData({ id: equTypeId, update: false });
  drawerApi.open();
}

async function handleEdit(record: Recordable<any>) {
  drawerApi.setData({ id: record.equTypeId, update: true });
  drawerApi.open();
}

async function handleDelete(row: Recordable<any>) {
  await delEquType(row.equTypeId);
  await tableApi.query();
}

function handleDownloadExcel() {
  commonDownloadExcel(equTypeExport, '设备类型', tableApi.formApi.form.values, {
    fieldMappingTime: formOptions.fieldMappingTime
  });
}

/**
 * 全部展开/折叠
 * @param expand 是否展开
 */
function setExpandOrCollapse(expand: boolean) {
  eachTree(tableApi.grid.getData(), (item) => (item.expand = expand));
  tableApi.grid?.setAllTreeExpand(expand);
}
</script>

<template>
  <Page :auto-content-height="true">
    <BasicTable table-title="菜单列表" table-title-help="双击展开/收起子菜单">
      <template #toolbar-tools>
        <Space>
          <a-button @click="setExpandOrCollapse(false)">
            {{ $t('pages.common.collapse') }}
          </a-button>
          <a-button @click="setExpandOrCollapse(true)">
            {{ $t('pages.common.expand') }}
          </a-button>
          <a-button v-access:code="['eims:equType:export']" @click="handleDownloadExcel">
            {{ $t('pages.common.export') }}
          </a-button>
          <a-button type="primary" v-access:code="['eims:equType:add']" @click="handleAdd">
            {{ $t('pages.common.add') }}
          </a-button>
        </Space>
      </template>
      <template #action="{ row }">
        <Space>
          <ghost-button v-access:code="['eims:equType:edit']" @click="handleEdit(row)">
            {{ $t('pages.common.edit') }}
          </ghost-button>
          <!-- '按钮类型'无法再添加子菜单 -->
          <ghost-button v-if="row.menuType !== 'F'" class="btn-success" v-access:code="['eims:equType:add']" @click="handleSubAdd(row)">
            {{ $t('pages.common.add') }}
          </ghost-button>
          <Popconfirm :get-popup-container="getVxePopupContainer" placement="left" title="确认删除？" @confirm="handleDelete(row)">
            <ghost-button danger v-access:code="['eims:equType:remove']" @click.stop="">
              {{ $t('pages.common.delete') }}
            </ghost-button>
          </Popconfirm>
        </Space>
      </template>
    </BasicTable>
    <EquTypeDrawer @reload="tableApi.query()" />
  </Page>
</template>
